/*
 * fast_graph_geo_mex.cc
 * ====================
 *
 * A mex wrapper around the fast_graph_geodesics.cc function
 *
 * To compile
 * ----------
 * 
 * mex -O -largeArrayDims fast_graph_geodesics_mex.cc fast_graph_geodesics.cc sf_util.cc
 *
 * 
 */
 
#include <mex.h>

#include "fast_graph_geodesics.h"


typedef unsigned int uint32;

uint32 get_uint32(const mxArray* arg) {
    mxAssert(mxGetClassID(arg)==mxUINT32_CLASS,"invalid uint32 class");
    return *((uint32*)mxGetPr(arg));
}
    

void mexFunction(int nargout, mxArray* pargout[],
                 int nargin, const mxArray* pargin[])
{
    // the function arguments are
    // nverts (uint32)
    // edges (uint32 2xnedges array)
    // nbytes (uint32)
    // nrep (uint32)Const
    // verbose (uint32)
    
    const mxArray* arg_nverts = pargin[0];
    const mxArray* arg_edges = pargin[1];
    const mxArray* arg_nbytes = pargin[2];
    const mxArray* arg_nrep = pargin[3];
    const mxArray* arg_verbose = pargin[4];        

    uint32 nverts = get_uint32(arg_nverts);
    
    mwSize nedges = mxGetN(arg_edges);
    if (mxGetM(arg_edges)!=2) {
        mexErrMsgIdAndTxt("fast_graph_geo_mex:invalidArg", 
                "the edges argument is not 2-by-nedges");
    }
    uint32* edges = (uint32*)mxGetPr(arg_edges);
                
    uint32 nbytes = get_uint32(arg_nbytes);
    uint32 nrep = get_uint32(arg_nrep);
    uint32 verbose = get_uint32(arg_verbose);
    
    nargout = 2;
    pargout[0] = mxCreateNumericMatrix(nverts, 1, mxUINT32_CLASS, mxREAL);
    pargout[1] = mxCreateNumericMatrix(nverts+1, 1, mxDOUBLE_CLASS, mxREAL);
    // we need the size of pargout[1] to be nverts+1 because we have to do
    // one additional iteration.  It simplifies the code dramatically
    // to have this one additional number
    
    fast_geodesics_estimate(nverts, (size_t)nedges, edges,
            (size_t)nbytes, (size_t)nrep, (size_t)nverts, NULL,
            (uint32*)mxGetPr(pargout[0]), mxGetPr(pargout[1]),
            NULL);
        
}
                  
