function [B perm] = largest_component(A)
% LARGEST_COMPONENT Output the largest connected component of the graph.
%
% [B perm] = largest_component(A)
% 
% Sets B as the largest connected component of the graph A.  
% B = A(perm, perm);
%
% If A is not square, the code assume you want the largest component of the
% bipartite graph [0 A; A' 0].  Then perm is a cell array where perm{1} is
% the permutation of the rows and perm{2} is the permutation of the
% columns.
%


if (size(A,1) ~= size(A,2))
    [m n] = size(A);
    B = [sparse(m,m) A; A' sparse(n,n)];
    c = components(B);
    
    S = sparse(1:(n+m), c, 1, n+m, max(c));

    [val maxc] = max(sum(S));

    perm = find(c == maxc);
    
    perm_r = find(perm <= m);
    perm_c = find(perm > m);
    perm = cell(2,1);
    perm{1} = perm_r;
    perm{2} = perm_c-m;
    
    B = B(perm_r, perm_c);
else
    c = components(A);
    n = size(A,1);

    S = sparse(1:n, c, 1, n, max(c));

    [val maxc] = max(sum(S));

    perm = find(c == maxc);
    B = A(perm, perm);

end;

